<?php
  /* vim: set expandtab tabstop=4 softtabstop=4 shiftwidth=4:
  Codificación: UTF-8
  +----------------------------------------------------------------------+
  | Elastix version 2.3.0-12                                               |
  | http://www.elastix.org                                               |
  +----------------------------------------------------------------------+
  | Copyright (c) 2006 Palosanto Solutions S. A.                         |
  +----------------------------------------------------------------------+
  | Cdla. Nueva Kennedy Calle E 222 y 9na. Este                          |
  | Telfs. 2283-268, 2294-440, 2284-356                                  |
  | Guayaquil - Ecuador                                                  |
  | http://www.palosanto.com                                             |
  +----------------------------------------------------------------------+
  | The contents of this file are subject to the General Public License  |
  | (GPL) Version 2 (the "License"); you may not use this file except in |
  | compliance with the License. You may obtain a copy of the License at |
  | http://www.opensource.org/licenses/gpl-license.php                   |
  |                                                                      |
  | Software distributed under the License is distributed on an "AS IS"  |
  | basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See  |
  | the License for the specific language governing rights and           |
  | limitations under the License.                                       |
  +----------------------------------------------------------------------+
  | The Original Code is: Elastix Open Source.                           |
  | The Initial Developer of the Original Code is PaloSanto Solutions    |
  +----------------------------------------------------------------------+
  $Id: index.php,v 1.1 2014-01-20 07:01:33 Alberto Santos asantos@palosanto.com Exp $ */
//include elastix framework
include_once "libs/paloSantoGrid.class.php";
include_once "libs/paloSantoForm.class.php";

function _moduleContent(&$smarty, $module_name)
{
    //include module files
    include_once "modules/$module_name/configs/default.conf.php";
    include_once "modules/$module_name/libs/paloSantoSmartassistant.class.php";

    //include file language agree to elastix configuration
    //if file language not exists, then include language by default (en)
    $lang=get_language();
    $base_dir=dirname($_SERVER['SCRIPT_FILENAME']);
    $lang_file="modules/$module_name/lang/$lang.lang";
    if (file_exists("$base_dir/$lang_file")) include_once "$lang_file";
    else include_once "modules/$module_name/lang/en.lang";

    //global variables
    global $arrConf;
    global $arrConfModule;
    global $arrLang;
    global $arrLangModule;
    $arrConf = array_merge($arrConf,$arrConfModule);
    $arrLang = array_merge($arrLang,$arrLangModule);

    //folder path for custom templates
    $templates_dir=(isset($arrConf['templates_dir']))?$arrConf['templates_dir']:'themes';
    $local_templates_dir="$base_dir/modules/$module_name/".$templates_dir.'/'.$arrConf['theme'];

    //conexion resource
    $pDB = new paloDB($arrConf['dsn_conn_database']);

    //actions
    $action = getAction();
    $content = "";

    switch($action){
	case "new_device": case "edit_device":
	    $content = newDevice($smarty, $module_name, $local_templates_dir, $pDB, $arrConf, $action);
	    break;
	case "save_device":
	    $content = saveDevice($smarty, $module_name, $local_templates_dir, $pDB, $arrConf);
	    break;
	case "delete_device":
	    $content = deleteDevice($smarty, $module_name, $local_templates_dir, $pDB, $arrConf);
	    break;
        default:
            $content = reportDevices($smarty, $module_name, $local_templates_dir, $pDB, $arrConf);
            break;
    }
    return $content;
}

function reportDevices($smarty, $module_name, $local_templates_dir, &$pDB, $arrConf)
{
    $pSmartassistant = new paloSantoSmartassistant($pDB);
    $filter_field = getParameter("filter_field");
    $filter_value = getParameter("filter_value");

    //begin grid parameters
    $oGrid  = new paloSantoGrid($smarty);
    $oGrid->setTitle(_tr("Smartassistant"));
    $oGrid->pagingShow(true); // show paging section.

    $oGrid->enableExport();   // enable export.
    $oGrid->setNameFile_Export(_tr("Smartassistant"));

    $url = array(
        "menu"         =>  $module_name,
        "filter_field" =>  $filter_field,
        "filter_value" =>  $filter_value);
    $oGrid->setURL($url);

    $arrColumns = array("",_tr("DeviceID"),_tr("Extension"),_tr("Allow Create Targets"));
    $oGrid->setColumns($arrColumns);

    $total   = $pSmartassistant->getTotalDevices($filter_field, $filter_value);
    $arrData = null;
    if($oGrid->isExportAction()){
        $limit  = $total; // max number of rows.
        $offset = 0;      // since the start.
    }
    else{
        $limit  = 20;
        $oGrid->setLimit($limit);
        $oGrid->setTotal($total);
        $offset = $oGrid->calculateOffset();
    }

    $arrResult =$pSmartassistant->getDevices($limit, $offset, $filter_field, $filter_value);

    if(is_array($arrResult) && $total>0){
        foreach($arrResult as $key => $value){ 
	    $arrTmp[0] = "<input type='checkbox' name='".$value['usuario']."' id='".$value['usuario']."'>";
	    $arrTmp[1] = "<a href=?menu=$module_name&action=edit_device&device={$value['usuario']}>".$value['usuario']."</a>";
	    $arrTmp[2] = $value['extension'];
	    if($value['target'] == 1)
	    	$arrTmp[3] = _tr("Yes");
	    else
		$arrTmp[3] = _tr("No");
            $arrData[] = $arrTmp;
        }
    }
    $oGrid->setData($arrData);
    $oGrid->setIcon("modules/$module_name/images/smart_assistant.png");
    $oGrid->addNew("new_device",_tr("Add Device"));
    $oGrid->deleteList(_tr("Are you sure you wish to delete the device(s).?"),"delete",_tr("Delete"));
    //begin section filter
    $oFilterForm = new paloForm($smarty, createFieldFilter());
    $smarty->assign("SHOW", _tr("Show"));
    $htmlFilter  = $oFilterForm->fetchForm("$local_templates_dir/filter.tpl","",$_POST);
    //end section filter

    $oGrid->showFilter(trim($htmlFilter));
    $content = $oGrid->fetchGrid();
    //end grid parameters

    return $content;
}

function saveDevice($smarty, $module_name, $local_templates_dir, &$pDB, $arrConf)
{
     $pSmartassistant = new paloSantoSmartassistant($pDB);
     $mode = getParameter("mode");
     if($mode == "edit_device")
	$deviceID = getParameter("deviceid");
     else
	$deviceID = getParameter("device");
     $extension = getParameter("extension");
     $target = getParameter("target");
     $error = FALSE;
     $arrFormNew = createFieldFormNew($pDB, $deviceID);
     $oForm      = new paloForm($smarty, $arrFormNew);
     if(!$oForm->validateForm($_POST)) {
        // Falla la validación básica del formulario
        $strErrorMsg = "<b>"._tr('The following fields contain errors').":</b><br/>";
        $arrErrores = $oForm->arrErroresValidacion;
        if(is_array($arrErrores) && count($arrErrores) > 0){
            foreach($arrErrores as $k=>$v) {
                $strErrorMsg .= "$k: [$v[mensaje]] <br /> ";
            }
        }
	$error = TRUE;
     }
     elseif(!preg_match("/^\w{16,320}$/",$deviceID)){
	 $strErrorMsg = _tr("The deviceID must contain at least 16 characters and only alphanumerics");
	 $error = TRUE;
     }
     elseif($mode != "edit_device" && count($pSmartassistant->getDeviceByID($deviceID)) > 0){
	 $strErrorMsg = _tr("The deviceID that you are entering already exists");
	 $error = TRUE;
     }
     elseif(!in_array($extension,$pSmartassistant->getExtensions($deviceID))){
	 $strErrorMsg = _tr("You must enter a valid extension");
	 $error = TRUE;
     }
     elseif(!in_array($target,array(1,0))){
	 $strErrorMsg = _tr("Invalid target value");
	 $error = TRUE;
     }

     if($error){
	$smarty->assign("mb_title", _tr("Validation Error"));
        $smarty->assign("mb_message", $strErrorMsg);
	return newDevice($smarty, $module_name, $local_templates_dir, $pDB, $arrConf, $mode);
     }
     else{
	 if($mode == "edit_device"){
		if(!$pSmartassistant->editDevice($deviceID,$extension,$target)){
			$smarty->assign("mb_title", _tr("ERROR"));
		        $smarty->assign("mb_message", $pSmartassistant->errMsg);
			$error = TRUE;
		}
         }
	 else{
		if(!$pSmartassistant->saveDevice($deviceID,$extension,$target)){
			$smarty->assign("mb_title", _tr("ERROR"));
                        $smarty->assign("mb_message", $pSmartassistant->errMsg);
			$error = TRUE;
		}
	 }
         if(!empty($_FILES["license_file"]["name"])){
		if(!preg_match("/^lic-\w{16,320}\.lic$/",$_FILES["license_file"]["name"])){
			$smarty->assign("mb_title", _tr("ERROR"));
                        $smarty->assign("mb_message", _tr("Invalid license file name"));
                        $error = TRUE;
		}
		elseif(!is_uploaded_file($_FILES['license_file']['tmp_name'])){
			$smarty->assign("mb_title", _tr("ERROR"));
                        $smarty->assign("mb_message", _tr('Possible file upload attack. Filename') ." :". $_FILES['license_file']['name']);
                        $error = TRUE;
		}
		elseif(strlen(trim(file_get_contents($_FILES['license_file']['tmp_name']))) != 1024){
			$smarty->assign("mb_title", _tr("ERROR"));
                        $smarty->assign("mb_message", _tr("Invalid license file"));
                        $error = TRUE;
		}else{
			if(file_exists("/opt/smartassistant/licencias/".$_FILES["license_file"]["name"]))
				rename("/opt/smartassistant/licencias/".$_FILES["license_file"]["name"],"/opt/smartassistant/licencias/".$_FILES["license_file"]["name"].".bak");
			move_uploaded_file($_FILES['license_file']['tmp_name'],"/opt/smartassistant/licencias/".$_FILES["license_file"]["name"]);
		} 
         }
	 if(!$error){
		$smarty->assign("mb_title", _tr("Message"));
                $smarty->assign("mb_message", _tr("Device successfully saved"));
	 }
	 return reportDevices($smarty, $module_name, $local_templates_dir, $pDB, $arrConf);
     }
}

function deleteDevice($smarty, $module_name, $local_templates_dir, &$pDB, $arrConf)
{
    $pSmartassistant = new paloSantoSmartassistant($pDB);
    $strError = "";
    foreach( $_POST as $key => $value ){
        if( $value == "on" ){
		if(!$pSmartassistant->deleteDevice($key))
			$strError .= $pSmartassistant->errMsg."<br />";	
	}
    }
    if(strlen($strError) == 0){
	$smarty->assign("mb_title", _tr("Message"));
        $smarty->assign("mb_message", _tr("Device(s) deleted correctly"));
    }
    else{
	$smarty->assign("mb_title", _tr("ERROR"));
        $smarty->assign("mb_message", $strError);
    }
    return reportDevices($smarty, $module_name, $local_templates_dir, $pDB, $arrConf);
}

function newDevice($smarty, $module_name, $local_templates_dir, &$pDB, $arrConf, $action)
{
    $pSmartassistant = new paloSantoSmartassistant($pDB);
    $smarty->assign("MODE",$action);
    $smarty->assign("CANCEL", _tr("Cancel"));
    $smarty->assign("SAVE", _tr("Save"));
    $smarty->assign("icon","modules/$module_name/images/smart_assistant.png");
    $deviceID = NULL;
    $_DATA = $_POST;
    if($action == "edit_device"){
	 $title = _tr("Edit Device");
	 $deviceID = getParameter("device");
	 $deviceData = $pSmartassistant->getDeviceByID($deviceID);
	 if(count($deviceData) == 0){
		$smarty->assign("mb_title", _tr("Validation Error"));
	        $smarty->assign("mb_message", _tr("The device ").$deviceID._tr(" does not exist"));
		return reportDevices($smarty, $module_name, $local_templates_dir, $pDB, $arrConf);
	 }
	 $smarty->assign("DEVICEID",$deviceID);
	 $_DATA["extension"] = $deviceData["extension"];
	 $_DATA["target"] = $deviceData["target"];
    }
    else{
	$title = _tr("New Device");
        $arrExtensions = $pSmartassistant->getExtensions();
	if(count($arrExtensions) <= 0){
		$smarty->assign("mb_title", _tr("Validation Error"));
                $smarty->assign("mb_message", _tr("You do not have available extensions. Click <a href='?menu=pbxadmin'>here</a> to create more."));
                return reportDevices($smarty, $module_name, $local_templates_dir, $pDB, $arrConf);
	}
    }
    $arrFormNew = createFieldFormNew($pDB, $deviceID);
    $oForm = new paloForm($smarty, $arrFormNew);
    $htmlForm = $oForm->fetchForm("$local_templates_dir/form.tpl", $title, $_DATA);
    $contenidoModulo = "<form  method='POST' enctype='multipart/form-data' style='margin-bottom:0;' action='?menu=$module_name'>".$htmlForm."</form>";

    return $contenidoModulo;
}


function createFieldFormNew(&$pDB, $deviceID)
{
     $pSmartassistant = new paloSantoSmartassistant($pDB);
     $arrExtensions = $pSmartassistant->getExtensions($deviceID);
     $arrTarget = array(1 => _tr("Yes"), 0 => _tr("No"));
     $arrFields = array(
            "extension" => array(   "LABEL"                  => _tr("Extension"),
                                    "REQUIRED"               => "yes",
                                    "INPUT_TYPE"             => "SELECT",
                                    "INPUT_EXTRA_PARAM"      => $arrExtensions,
                                    "VALIDATION_TYPE"        => "numeric",
                                    "VALIDATION_EXTRA_PARAM" => "" ),
            "target"  => array(   "LABEL"                  => _tr("Allow Create Targets"),
                                    "REQUIRED"               => "yes",
                                    "INPUT_TYPE"             => "SELECT",
                                    "INPUT_EXTRA_PARAM"      => $arrTarget,
                                    "VALIDATION_TYPE"        => "text",
                                    "VALIDATION_EXTRA_PARAM" => ""),
	    "device"  => array(   "LABEL"                    => _tr("DeviceID"),
                                    "REQUIRED"               => "no",
                                    "INPUT_TYPE"             => "TEXT",
                                    "INPUT_EXTRA_PARAM"      => array("style" => "width:176px"),
                                    "VALIDATION_TYPE"        => "text",
                                    "VALIDATION_EXTRA_PARAM" => ""),
 	"license_file"  => array(   "LABEL"                  => _tr("License File"),
                                    "REQUIRED"               => "no",
                                    "INPUT_TYPE"             => "FILE",
                                    "INPUT_EXTRA_PARAM"      => "",
                                    "VALIDATION_TYPE"        => "text",
                                    "VALIDATION_EXTRA_PARAM" => ""),
	);
    return $arrFields;
}

function createFieldFilter(){
    $arrFilter = array(
	    "usuario" => _tr("DeviceID"),
	    "extension" => _tr("Extension"),
	    "target" => _tr("Allow Create Targets")
                    );

    $arrFormElements = array(
            "filter_field" => array("LABEL"                  => _tr("Search"),
                                    "REQUIRED"               => "no",
                                    "INPUT_TYPE"             => "SELECT",
                                    "INPUT_EXTRA_PARAM"      => $arrFilter,
                                    "VALIDATION_TYPE"        => "text",
                                    "VALIDATION_EXTRA_PARAM" => ""),
            "filter_value" => array("LABEL"                  => "",
                                    "REQUIRED"               => "no",
                                    "INPUT_TYPE"             => "TEXT",
                                    "INPUT_EXTRA_PARAM"      => "",
                                    "VALIDATION_TYPE"        => "text",
                                    "VALIDATION_EXTRA_PARAM" => ""),
                    );
    return $arrFormElements;
}


function getAction()
{
    if(getParameter("save")) //Get parameter by POST (submit)
        return "save_device";
    else if(getParameter("action") == "edit_device")
        return "edit_device";
    else if(getParameter("new_device")) 
        return "new_device";
    else if(getParameter("delete"))
	return "delete_device";
    else
        return "report"; //cancel
}
?>
